%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% This script is used to obtain the results in the paper
%%%% "Tree based compressed sensing MRI"
%%%%% Jan. 21, 2013, Created by Chen Chen and Junzhou Huang in University
%%%%% of Texas at Arlington
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; %clc;
randn('state',0);rand('state',0);

F = double(imread('pics/Heart.png'));

F=imresize(F, [128 128]);[m n] = size(F); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2D wavelet transformation and sparse signal -- theta0 
DecLevel=3;
WaveletName='db1';
[C0, S0] = wavedec2(F, DecLevel, WaveletName);
[IdxParent, IdxChildren, Ms]=WaveRelation2D(C0, S0);
theta0=C0(:);

[G,Gmat,GmatX,groups]=GroupMatrics(IdxParent');
input.G=G;input.Gt=G';input.GtG=G'*G;
input.groups=groups;input.Gmat=Gmat;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
input.no=50; iter=0;Maxiter = 3;
snr1=zeros(Maxiter,10);snr2=snr1;snr3=zeros(Maxiter,input.no);snr4=snr3;snr5=snr3;
times1=snr1;times2=snr1;times3=snr3;times4=snr3;times5=snr3;
for iter=1:Maxiter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Preparing the data and operators
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N = m*n;   %%%% choose 3, 4, 
d=4.4; tol=10e-10;
[OMEGA] = RandMask_rect(double(m/d),double(n/d),m,n);
% Number of projections
k = length(OMEGA);
[mask] = RandMask_InverseTransfer(OMEGA,m,n); 
% partial FFT tranform
R = @(x) A_fhp_rect(x, OMEGA, m, n);
RT = @(x) At_fhp_rect(x, OMEGA, m, n);
% define the function handles that compute 
% the products by W (inverse DWT) and W' (DWT)
WT = @(x) wavedec2(reshape(x,m,n),DecLevel,WaveletName)';
W = @(x) reshape(waverec2(x', S0, WaveletName),N,1); 
AO = @(x) R(W(x)); AOT = @(x) WT(RT(x)); 
A = A_operator(@(x) AO(x), @(x) AOT(x));
Phi = A_operator(@(x) WT(x), @(x) W(x));    % notice Phi=WT
RR = A_operator(@(x) R(x), @(x) RT(x));
 
F=F/255;f = F(:);
sigma = 0.01; noise = sigma*randn(2*k+1,1);
b = R(f) + noise;v=R(theta0)+noise;
alpha = 0.01; 
beta = 3.5e-2; 
input.lamda=beta*0.2;
%alpha=0;
xs = WT(f);input.f=f; input.xs=xs;
input.n1=m;input.n2=n;
input.alpha=alpha;input.beta=beta;
input.A=A;input.Phi=Phi;%input.b=v;
input.maxitr=500;input.maxiitr= 500;
input.stopCriterion=2; 
input.tolA=tol;
input.L=1;input.num=1; 
% input.l=-inf; input.u=inf;
input.l=0; input.u=255;input.b=b;
input2=input; input2.A=RR;input2.b=b;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% turboAMP 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('calling the function turboAMP.....\n');
%M=5000;             % number of measurements
SNRdB_true=Inf;% SNR  in dB 
%numLevels=4;   % wavelet tree depth 
A_type = 2;    % Measurement matrix: 1=iid Gaussian, 2=Rademacher
dim=size(F);  % dimension of the image
N=prod(dim);  % total number of pixels
%%% settings for turbo AMP algorithm
params.projOnImage=1;  % 0: proj on wavelet trandsform, 1: proj on image
params.iterTurbo=10;   % number of turbo iterations
params.tolTurbo=tol;  % tolerance for convergnece of turbo
params.warm=0;         % warm start for turbo
params.tolAMP=tol;    % tolerance for convergence of AMP
params.iterAMP=10;     % number of AMP iterations
params.gmm=0;          % 0: Bernoulli-Gaussian model, 1: Gaussian mixture model
params.Le_max=20;      % maximum value of extrinsic log-likelihood 
params.numLevels=DecLevel;
params.s=S0;
modelParams.Gamma=[1 1e-6];
params.f=f;

javaaddpath('./turboAMP/hmt/hmt_struct');
if exist('modelParams','var')
    [modelParams, ff]=initializeModelParams(params, modelParams);
else
    [modelParams, ff]=initializeModelParams(params);
end
out=MturboAMP(b,RR,params,modelParams,ff);
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f \n',out.xtime(end), params.iterTurbo, out.snr(end));
out1=out;
im1= out1.x;
SNR1 = snr(im1, input.f);
im1=reshape(im1, [m, n]);
fprintf(1,'SNR=%f\n',SNR1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MTSWCSvb
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hyperpara=[];VBpara.tol=tol; VBpara.maxIter=10;
fprintf('calling the function VB.....\n');
out = MTSWCSvb(A, b, Ms, IdxParent', IdxChildren',W,f,hyperpara,VBpara);
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f \n',out.xtime(end), input.no, out.snr(end));
out2=out;x=out.x;im2=Phi'*x;
SNR2 = snr(im2, input.f);
im2=reshape(im2, [m, n]);
fprintf(1,'SNR=%f\n',SNR2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% slep
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
F=F*255;f=F(:);b = R(f) + noise;
fprintf('calling the function slep_overlap.....\n');
tol=1e-10;
opts=[];opts.n=N;
% Starting point
opts.init=2;       
% Termination 
opts.tFlag=3;       % the relative change is less than opts.tol
opts.maxIter=input.no;  % maximum number of iterations
opts.tol=tol;      % the tolerance parameter
% regularization
opts.rFlag=1;       % use ratio
% Normalization
opts.nFlag=0;       % without normalization
Gr=G*(1:N)';w = zeros(2,N);
for i=1:groups(end)
    ii = find(groups==i);
    if length(ii)==1
        w(1,i)=ii;
        w(2,i)=ii;
    else
        w(1,i)=ii(1);
        w(2,i)=ii(end);
    end
end
w =w;
w=[w;ones(1,N)];
opts.G=Gr;
opts.ind=w;
opts.rStartNum=100;
z=[0,beta];opts.Phi=Phi;
opts.maxIter2=1;
opts.tol2=tol;
opts.flag2=2;opts.f=f;
[x, out]= overlapping_LeastR(A, b, z, opts);
im3 = Phi'*x;
SNR1 = snr(im3,f);out3=out;im3=reshape(im3, [m, n]);
fprintf(1,'SNR=%f\n',SNR1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% YALL1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('calling the function YALL1.....\n');
opts.tol = tol;opts.f=f;
opts.weights= ones(N,1);
opts.nonneg=0;opts.Phi=Phi;
opts.maxit =input.no;
 opts.beta=[beta,beta];  % must be a 2-vector
opts.gamma = [1.618, 1.618];
opts.continuation=0;
AA = @(x) A*x; AAt = @(x) A'*x;
[x,out] = overlap_primal_solver(AA,AAt,b,groups,Gmat,G,opts);
im4 = Phi'*x;
SNR2 = snr(im4,f);out4=out;im4=reshape(im4, [m, n]);
fprintf(1,'SNR=%f\n',SNR2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% WaTMRI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
input2.b=b;input2.f=f;
fprintf('calling the function WaTMRI.....\n');
out = WaTMRI(input2);  % matlab wavelet
out5=out;im5= out.y; x=Phi*im5; im5=reshape(im5, [m, n]);
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f \n',out.xtime(end), input.no, out.snr(end));
% fprintf(1,'k = %d, rec_err = %10.6e, samp.ratio = %f, func.value = %f\n\n',...
% %     k, norm(x-xs)/norm(xs),k/(m*n),out.funv(end));
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
snr1(iter,:)=out1.snr;snr2(iter,:)=out2.snr;snr3(iter,:)=out3.snr;snr4(iter,:)=out4.snr;snr5(iter,:)=out5.snr;
times1(iter,:)=out1.xtime;times2(iter,:)=out2.xtime;times3(iter,:)=out3.xtime;times4(iter,:)=out4.xtime;times5(iter,:)=out5.xtime;
end
SNR1 = mean(snr1,1);SNR2 = mean(snr2,1);SNR3 = mean(snr3,1);SNR4 = mean(snr4,1);SNR5 = mean(snr5,1);
Time1=mean(times1,1);Time2=mean(times2,1);Time3=mean(times3,1);Time4=mean(times4,1);Time5=mean(times5,1);
SNR1_S = std(snr1,0,1);SNR2_S = std(snr2,0,1);SNR3_S = std(snr3,0,1);SNR4_S = std(snr4,0,1);SNR5_S = std(snr5,0,1);
Time1_S=std(times1,0,1);Time2_S=std(times2,0,1);Time3_S=std(times3,0,1);Time4_S=std(times4,0,1);Time5_S=std(times5,0,1);

titlename=num2str(1);  lw = 3;
h=figure; hold on;box on;
set(h,'name',titlename,'Numbertitle','off');
plot(SNR1, 'k:','linewidth', lw);
plot(SNR2, 'g+-','linewidth', lw);
plot(SNR3, 'r--','linewidth', lw);
plot(SNR4, 'm-.','linewidth', lw);
plot(SNR5, 'b-','linewidth', lw);
legend('AMP','VB','SLEP','YALL1','WaTMRI','Location','SouthEast');
xlabel('Iterations');
ylabel('SNR');
 textobj = findobj('type', 'text');
set(textobj, 'fontsize', 14);
h_xlabel = get(gca,'XLabel');
set(h_xlabel,'FontSize',14); 
h_xlabel = get(gca,'YLabel');
set(h_xlabel,'FontSize',14); 

titlename=num2str(2); 
h=figure; hold on;box on;
set(h,'name',titlename,'Numbertitle','off');
plot(Time1(1:10),SNR1(1:10), 'k:','linewidth', lw);
% plot(Time2,SNR2, 'g-.','linewidth', lw);
plot(Time3,SNR3, 'r--','linewidth', lw);
plot(Time4 ,SNR4, 'm-.','linewidth', lw);
plot(Time5,SNR5, 'b-','linewidth', lw);

legend('AMP','SLEP','YALL1','WaTMRI','Location','SouthEast');
xlabel('CPU Time (S)');
ylabel('SNR');
 textobj = findobj('type', 'text');
set(textobj, 'fontsize', 14);
h_xlabel = get(gca,'XLabel');
set(h_xlabel,'FontSize',14); 
h_xlabel = get(gca,'YLabel');
set(h_xlabel,'FontSize',14); 

 figure; hold on;
 subplot(2,3,1); imshow(F, []); title('Original');
 subplot(2,3,2); imshow(im1, []); title('AMP');
 subplot(2,3,3); imshow(im2, []); title('VB');
 subplot(2,3,4); imshow(im3, []); title('SLEP');
 subplot(2,3,5); imshow(im4, []); title('YALL1');
 subplot(2,3,6) ;imshow(im5, []); title('WaTMRI');


