%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% This script is used to obtain the results in the paper
%%%% "Tree based compressed sensing MRI"
%%%%% Jan. 21, 2013, Created by Chen Chen and Junzhou Huang in University
%%%%% of Texas at Arlington
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all;

addpath(strcat(pwd,'/utils'));

temp = zeros([256,256,8]);
load rawdata_brain;

c=0;
 
alpha = 0.001; 
beta = 0.035; 
lamda = 0.025;
for k = 1:8   % from channel 1 to 8
fprintf('Channel %d-------------------------------------------------\n',k); 

temp(:,:,k) = fftshift(ifft2(fftshift(raw_data(:,:,k))));

F = temp(:,:,k);
F = (F - min(abs(F(:))))/( max(abs(F(:)))- min(abs(F(:))));
F = F*255;
temp(:,:,k) = F;
ff(:,k) = F(:); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L1 Recon Parameters 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m,n] = size(F);
N = size(F); 	% image Size
DN = size(F); 	% data Size
Itnlim = 200;   % Number of iterations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sampling Mask
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
d = 3.8;
[OMEGA] = RandMask_rect(double(m/d),double(n/d),m,n);
[mask] = RandMask_InverseTransfer(OMEGA,m,n); 

sigma = 0.01;
%generate Fourier sampling operator
FT = p2DFT(mask, N, 1, 2);
data = FT*F;
data = data.*mask+sigma*randn(m,n)+1i*sigma*randn(m,n);

im_dc = FT'*(data);
im_dc = im_dc/max(abs(im_dc(:)));

%generate transform operator
XFM = Wavelet('Daubechies',4,3);	% Wavelet

%initialize Parameters for reconstruction
param = init;
param.FT = FT;
param.XFM = XFM;
param.TV = TVOP;
param.data = data.*mask;
param.TVWeight =alpha;     % TV penalty 
param.xfmWeight = beta;  % L1 wavelet penalty
param.Itnlim = Itnlim/2;
param.f = F;

res = XFM*im_dc;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CG
% do iterations
fprintf('calling the function CG.....\n');
[res,snrtrace,timetrace] = fnlCg(res,param);
im_res = XFM'*res;

temp1(:,:,k) = im_res;
SNR1(:,k) = snrtrace;
TIME1(:,k) = timetrace;

fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f \n',timetrace(end), param.Itnlim, snrtrace(end));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FCSA
% do iterations
input.f=abs(F);
input.n1=N(1);
input.n2=N(2);
input.alpha=alpha;input.beta=beta;
input.Phi=XFM;
input.maxitr=5000;input.maxiitr= 5000;
input.stopCriterion=2; 
input.tolA=1e-8; input.no=Itnlim; 
input.L=1;input.num=1; 
input.l=-inf; input.u=inf;
input.b=data;input.A=FT;

fprintf('calling the function FCSA.....\n');
out = TVS_FCSAy2D(input);
out3=out;im3 = out.y; im3=reshape(im3, [m, n]);
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f \n',out.xtime(end), input.no, out.snr(end));

snr3(k) = snr(abs(im3),abs(F));
temp3(:,:,k) = im3;
SNR3(:,k) = out.snr;
TIME3(:,k) = out.xtime;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% WaTMRI
DecLevel=4;
[IdxParent, IdxChildren, Ms] = WaveTreeStructure2D(m,DecLevel);
[G,Gmat,GmatX,groups]=GroupMatrics(IdxParent);
input.G=G;input.Gt=G';input.GtG=G'*G;
input.groups=groups;input.Gmat=Gmat;
input.lamda=lamda;

fprintf('calling the function WaTMRI.....\n');
out = WaTMRI2D(input);
out4=out;im4 = out.y; im4=reshape(im4, [m, n]);
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f \n',out.xtime(end), input.no, out.snr(end));
snr4(k) = snr(abs(im4),abs(F));
SNR4(:,k) = out.snr;
temp4(:,:,k) = im4;
TIME4(:,k) = out.xtime;
end

MTSNR1 = MT_SNR(SNR1, ff);
MTSNR3 = MT_SNR(SNR3, ff);
MTSNR4 = MT_SNR(SNR4, ff);
TIME1 = mean(TIME1,2);
TIME3 = mean(TIME3,2);
TIME4 = mean(TIME4,2);

lw=3;
figure;hold on;box on;
plot(TIME1,MTSNR1,'m-.','linewidth', lw);
plot(TIME3,MTSNR3,'r--','linewidth', lw);
plot(TIME4,MTSNR4,'b-','linewidth', lw);
legend('CG','FCSA','WaTMRI','Location','SouthEast');
xlabel('CPU Time (S)');
ylabel('SNR');
% 
textobj = findobj('type', 'text');
set(textobj, 'fontsize', 14);
h_xlabel = get(gca,'XLabel');
set(h_xlabel,'FontSize',14); 
h_xlabel = get(gca,'YLabel');
set(h_xlabel,'FontSize',14); 


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sum of squares

c = 0;
for k=1:8
    a=real(temp(:,:,k));
    b=imag(temp(:,:,k));
    c=a.*a+b.*b+c;
end
F0=sqrt(c);


c = 0;
for k=1:8
    a=real(temp3(:,:,k));
    b=imag(temp3(:,:,k));
    c=a.*a+b.*b+c;
end
im3=sqrt(c);

c = 0;
for k=1:8
    a=real(temp4(:,:,k));
    b=imag(temp4(:,:,k));
    c=a.*a+b.*b+c;
end
im4=sqrt(c);

c = 0;
for k=1:8
    a=real(temp1(:,:,k));
    b=imag(temp1(:,:,k));
    c=a.*a+b.*b+c;
end
im1=sqrt(c);

snr1 = snr(abs(im1),abs(F0))
snr3 = snr(abs(im3),abs(F0))
snr4 = snr(abs(im4),abs(F0))


err0 = norm(im1(:)-F0(:))/norm(F0(:))
err1 = norm(im3(:)-F0(:))/norm(F0(:))
err2 = norm(im4(:)-F0(:))/norm(F0(:))


figure, imshow(abs(cat(2,im1,im3,im4)),[]);
%figure, imshow(abs(cat(2,im1-F0,im3-F0,im4-F0)),[]);

im(:,:,1) = F0;
im(:,:,2) = im1;
im(:,:,3) = im3;
im(:,:,4) = im4;
out = zoominim(im);
figure;imshow(out,[]);