% This Matlab script demonstrates the Turbo AMP reconstruction
% for Compressive Imaging. 
% 
% turboAMP version 1.0 
% Copyright (c) Subhojit Som and Philip Schniter, 2011
% Email: som.4@osu.edu, schniter@ece.osu.edu
%
% Last modified: July 30, 2011



close all; clear all; clear java; clc;


path(path,'utils/');
path(path,'hmt/');
path(path,'turbo/');


%%% read image file %%% 
% X=imread('cameraman.tif');
% X=X(21:148,71:198);
% X=double(X)/255;
% X=imresize(X,0.5,'bicubic');

load imageData.mat; % This directly loads image as variable X. May use
                    % the previous lines to load the image as well and
		            % comment this line.
M=5000;             % number of measurements



SNRdB_true=Inf;% SNR  in dB 
numLevels=4;   % wavelet tree depth 
A_type = 1;    % Measurement matrix: 1=iid Gaussian, 2=Rademacher



dim=size(X);  % dimension of the image
N=prod(dim);  % total number of pixels




%%% settings for turbo AMP algorithm
params.projOnImage=1;  % 0: proj on wavelet trandsform, 1: proj on image
params.iterTurbo=10;   % number of turbo iterations
params.tolTurbo=1e-5;  % tolerance for convergnece of turbo
params.warm=0;         % warm start for turbo
params.tolAMP=1e-5;    % tolerance for convergence of AMP
params.iterAMP=10;     % number of AMP iterations
params.gmm=0;          % 0: Bernoulli-Gaussian model, 1: Gaussian mixture model
params.Le_max=20;      % maximum value of extrinsic log-likelihood 


tryGenA=true;
k=1;
while (tryGenA)
    try % see if the computer can support this problem size
        if A_type==1, % iid Gaussian
            A = (randn(M,N))/sqrt(M);     
        elseif A_type==2, % Rademacher
            A = sign(randn(M,N))/sqrt(M);
        end
    catch err % if not, then reduce the problem size
	if (strcmp(err.identifier,'MATLAB:nomem'))
	    M=round(M/2);
        X=img{k};
        dim=size(X);  
        N=prod(dim); 
	    k=k+1;
	    display('Reduced problem size since measurement matrix could not fit into memory!')
            continue;
	end
    end
    tryGenA=false;
end

for n=1:N
    A(:,n) = A(:,n)/norm(A(:,n)); 
end

%%% wavelet trandform of the image %%%
[waveC, signature]=wavedec2(X,numLevels,'db1');
W=wave_vec2mat(waveC,signature);
params.numLevels=numLevels;
params.s=signature;


if(params.projOnImage) 
    x_true=X(:);
else
    x_true=W(:);
end
y = A*x_true;

sig2w_true=norm(y)^2/M*10^(-SNRdB_true/10);     
w=sqrt(sig2w_true)*randn(M,1);
y=y+w;

params.f = x_true;
%%% INITIALIZATION %%%
% In this example noise Gamma parameters are set by the following line. If
% commented, the default value [1 1e-6] will be used.
modelParams.Gamma=[1 1e-6]; % noise Gamma params [a b]

% Similarly, the following parameters can be set by the user. If nothing is
% set the default values mentioned in the paper are used.

% modelParams.Gamma_j= ...;    % [(numLevels+1)x2, [a_j b_j] per row, starting at j=-1]
% modelParams.Beta_0_1= ...;   % [1x2 double, [c d]]
% modelParams.Beta_j_11= ...;  % [(numLevels-1)x2 [c_j d_j] per row, starting at j=0]
% modelParams.Beta_j_00= ...;  % [(numLevels-1)x2 [\underbar{c}_j \underbar{d}_j] per row, starting at j=0]
% modelParams.Beta_approx= ...;% [1x2, [\underbar{c} \underbar{d}]]

if exist('modelParams','var')
    [modelParams, f]=initializeModelParams(params, modelParams);
else
    [modelParams, f]=initializeModelParams(params);
end


[out]=MturboAMP(y,A,params,modelParams,f);

figure;plot(out.snr);
if(params.projOnImage)
    figure(1);
    % if you do not have image processing toolbox, comment the following
    % two lines and uncomment the next two lines
    subplot(121);imshow(reshape(x_true,dim));colormap('gray');title('original image');axis square;
    subplot(122);imshow(reshape(x_hat,dim));colormap('gray');title('reconstructed image');axis square;
%     subplot(121);imagesc(reshape(x_true,dim));colormap('gray');title('original image');axis square;
%     subplot(122);imagesc(reshape(x_hat,dim));colormap('gray');title('reconstructed image');axis square;
else
    figure;
    subplot(211);plot(x_true);title('original wavelet coefficients');
    subplot(212);plot(x_hat,'r');title('reconstructed wavelet coefficients');
end




